import numpy as np
from matplotlib import pyplot as plt
from PIL import Image
import sys

sys.path.insert(0, '../../algorithms/distance/')

from rmsdiff import rmsdiff

sys.path.insert(0, '../../algorithms/context_adaptive_interpolator/')

from context_adaptive_interpolator import contextAdaptiveInterpolator
from tqdm import tqdm

IMAGE_SIZE = 64
NUMBER_OF_PHONES = 10
NUMBER_OF_IMAGES_PER_PHONE = 100
# Compared to images.
PRNU_FACTOR = 0.1

# Is such `np.maximum` probabilistically correct with our theoretical method? See #19.
def randomImage(scale):
    return np.maximum(np.random.normal(loc = 0, scale = scale, size = (IMAGE_SIZE, IMAGE_SIZE)), 0)

prnus = [randomImage(scale = PRNU_FACTOR) for _ in range(NUMBER_OF_PHONES)]

images = [[randomImage(scale = 1) + prnus[phoneIndex] for _ in range(NUMBER_OF_IMAGES_PER_PHONE)] for phoneIndex in range(NUMBER_OF_PHONES)]

# How to simplify?
allImages = np.max([np.max(prnus) + np.max(images)])

def toPilImage(npArray):
    npArray = (255 * npArray / np.max(allImages)).round()
    return Image.fromarray(np.uint8(npArray))

prnus = [toPilImage(prnu) for prnu in prnus]
#prnus[0].show()

images = [[toPilImage(image) for image in images[phoneIndex]] for phoneIndex in range(NUMBER_OF_PHONES)]
#images[0][0].show()

caiImages = [[contextAdaptiveInterpolator(image.load(), image) for image in images[phoneIndex]] for phoneIndex in tqdm(range(NUMBER_OF_PHONES))]

def getPhoneIndexByNearestPrnu(estimatedPrnu):
    nearestPrnuRmsDiff = None
    nearestPrnuPhoneIndex = None
    for phoneIndex, prnu in enumerate(prnus):
        prnuRmsDiff = rmsdiff(estimatedPrnu, prnu)
        if nearestPrnuRmsDiff is None or prnuRmsDiff < nearestPrnuRmsDiff:
            nearestPrnuRmsDiff = prnuRmsDiff
            nearestPrnuPhoneIndex = phoneIndex
    return nearestPrnuPhoneIndex

# Should choose between *average* and *mean*, I would prefer *mean*.
# What about CAI on `phoneImagesMean`?
correctGuessesByAverageImages = 0
correctGuessesByAverageCAIImages = 0
for phoneIndex in range(NUMBER_OF_PHONES):
    phoneImages = images[phoneIndex]
    phoneCaiImages = caiImages[phoneIndex]

    phoneImagesMean = toPilImage(np.array(phoneImages).mean(axis = 0))
    caiImagesMean = toPilImage(np.array(phoneCaiImages).mean(axis = 0))
    phonePrnu = prnus[phoneIndex]
    print('RMS diff with average image =', rmsdiff(phoneImagesMean, phonePrnu))
    print('RMS diff with average CAI images =', rmsdiff(caiImagesMean, phonePrnu))
    guessedPhoneIndexByAverageImages = getPhoneIndexByNearestPrnu(phoneImagesMean)
    guessedPhoneIndexByAverageCAIImages = getPhoneIndexByNearestPrnu(caiImagesMean)
    print(f'Actual phone index {phoneIndex}, guessed phone index {guessedPhoneIndexByAverageImages} by averages images and {guessedPhoneIndexByAverageCAIImages} by average CAI images')
    correctGuessesByAverageImages += 1 if phoneIndex == guessedPhoneIndexByAverageImages else 0
    correctGuessesByAverageCAIImages += 1 if phoneIndex == guessedPhoneIndexByAverageCAIImages else 0

print(f'{correctGuessesByAverageImages / NUMBER_OF_PHONES=}')
print(f'{correctGuessesByAverageCAIImages / NUMBER_OF_PHONES=}')

